/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/rp/ClassifierAdaptors.h
  \brief Classifier Adaptors.
 */

#ifndef __TERRALIB_RP_INTERNAL_CLASSIFIERADAPTORS_H
#define __TERRALIB_RP_INTERNAL_CLASSIFIERADAPTORS_H

#include "Config.h"
#include "../classification/Adaptors.h"
#include "../raster/Raster.h"

namespace te
{
  namespace rp
  {
    namespace ca
    {
      /*!
        \class RasterInputAdaptor
        \brief Classifiers raster input data adaptor.

      */
      template<typename DataType>
      class RasterInputAdaptor : public te::cl::InputAdaptor< DataType >
      {
        public:

          RasterInputAdaptor( const te::rst::Raster& raster )
          : m_raster( raster ), m_nRows( raster.getNumberOfRows() ),
            m_nCols( raster.getNumberOfColumns() )
          {};

          ~RasterInputAdaptor() {};

          // overload
          void getFeature(const unsigned int& elementIndex, const unsigned int& featureIndex,
            DataType& featureValue ) const
          {
            m_raster.getValue( elementIndex % m_nCols, elementIndex / m_nCols, featureValue, featureIndex );
          };

          // overload
          unsigned int getElementsCount() const
          {
            return ( m_nRows * m_nCols );
          };

          // overload
          unsigned int getFeaturesCount() const
          {
            return static_cast<unsigned int>(m_raster.getNumberOfBands());
          };

        protected :

          const te::rst::Raster& m_raster;
          unsigned int m_nRows;
          unsigned int m_nCols;
      };

      /*!
        \class RasterOutputAdaptor
        \brief Classifiers raster output data adaptor.

      */
      template<typename DataType>
      class RasterOutputAdaptor : public te::cl::OutputAdaptor< DataType >
      {
        public:

          RasterOutputAdaptor( te::rst::Raster& raster )
          : m_raster( raster ), m_nRows( raster.getNumberOfRows() ),
            m_nCols( raster.getNumberOfColumns() )
          {};

          ~RasterOutputAdaptor() {};

          // overload
          void setFeature(const unsigned int& elementIndex,
            const unsigned int& featureIndex, const DataType& value )
          {
            m_raster.setValue( elementIndex % m_nCols, elementIndex / m_nCols, (double)value,
              featureIndex );
          };

          // overload
          unsigned int getElementsCount() const
          {
            return ( m_nRows * m_nCols );
          };

          // overload
          unsigned int getFeaturesCount() const
          {
            return static_cast<unsigned int>(m_raster.getNumberOfBands());
          };

        protected :

          te::rst::Raster& m_raster;
          unsigned int m_nRows;
          unsigned int m_nCols;
          double m_value;
      };
    }  // end namespace ca
  } // end namespace rp
}   // end namespace te

#endif  // __TERRALIB_RP_INTERNAL_CLASSIFIERADAPTORS_H

