/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/layout/qt/core/tools/CreateItemTool.h

  \brief This class implements a concrete tool to create item.
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_CREATE_ITEM_TOOL_H
#define __TERRALIB_LAYOUT_INTERNAL_CREATE_ITEM_TOOL_H

#include "RubberBandTool.h"
#include "../../../core/property/Properties.h"

//TerraLib
#include "terralib/geometry/Coord2D.h"

//STL
#include <vector>
#include <string>

// Qt
class QMouseEvent;
class QGraphicsItem;
class QStringList;

namespace te
{
  namespace layout
  {
    // Forward declarations
    class View;
    class EnumType;
    class Scene;
      /*!
        \class CreateLineItemTool

        \brief This class implements a concrete tool to create line in a QGraphics Item.
      */
    class TELAYOUTEXPORT CreateItemTool : public RubberBandTool
    {
        public:

          /** @name Initializer Methods
           *  Methods related to instantiation and destruction.
           */
          //@{

          /*!
            \brief It constructs a create line item tool associate with a View.

            \param view The tool's view.
            \param parent The tool's parent..
          */
          CreateItemTool(View* view, EnumType* itemType, QObject* parent = 0);

          /*! \brief Destructor. */
          ~CreateItemTool();

          //@}

          /** @name AbstractLayoutTool Methods
           *  Methods related with tool behavior.
           */
          //@{

          bool mousePressEvent(QMouseEvent* e);
          
          bool mouseMoveEvent(QMouseEvent* e);

          virtual bool mouseReleaseEvent(QMouseEvent* e);

          void setProperties(const te::layout::Properties& properties);

          Properties getProperties();

          void reset();
          
        protected:

          void addProperties(te::layout::Properties& properties, const QSize& size, const te::gm::Coord2D& coord);

          /*!
            \brief Associate with a map item.

            \param item component to associate
            \param scene responsible for managing items QGraphicsItem and know when and how to redrawing them
          */
          virtual bool associateToMap(QGraphicsItem* item, Scene* scene);

          /*!
            \brief Search for a map item and associate it with the created item. If more than one map is found, the user is asked which one to choose.

            \param newItem created item
            \param scene responsible for managing items QGraphicsItem and know when and how to redrawing them
          */
          virtual bool canAssociateItem(QGraphicsItem* newItem, Scene* scene);

          /*!
            \brief Search for a map item by name.

            \param nameItem map item name to search
            \param scene responsible for managing items QGraphicsItem and know when and how to redrawing them
          */
          virtual QGraphicsItem* searchMap(const std::string& nameItem, Scene* scene);

          /*!
            \brief Get all map items name.

            \param scene responsible for managing items QGraphicsItem and know when and how to redrawing them
          */
          virtual QStringList getAllMapItemsNames(Scene* scene);

          /*!
            \brief Ask user which map item will choose.

            \param mapNames all map items names
            \param scene responsible for managing items QGraphicsItem and know when and how to redrawing them
          */
          QString showInputDialog(const QStringList& mapNames, Scene* scene);

          //@}
        protected:
          
          EnumType*      m_itemType;
          Properties     m_properties;
          QGraphicsItem* m_mapItem; //<! map item found
      };
  }   // end namespace layout
}     // end namespace te

#endif  
